/*
   fenris - program execution path analysis tool
   ---------------------------------------------

   Copyright (C) 2001, 2002 by Bindview Corporation
   Portions copyright (C) 2001, 2002 by their respective contributors
   Developed and maintained by Michal Zalewski <lcamtuf@coredump.cx>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   This file was extracted from fenris.c by Marcin Gozdalik :-)

   Here are my malloc() and realloc() versions that return cleared
   memory, plus do few other things, like tracking of buffers,
   return value checks.

 */


#include <string.h>

#define USE_ORIGINAL_ALLOCS
#include "allocs.h"

#ifdef DEBUG
static void* mchunks[MAXCHUNKS+1];
static unsigned int mtop;
static unsigned int memop;
#endif /* DEBUG */


void* my_malloc(const int size) {
  void* r;
  int siz;
  if (size<=0) fatal("my_malloc size is <=0",0);
  r=calloc(1,size);
  if (!r) fatal("calloc failed",0);
  siz=malloc_usable_size(r);
  if (size<siz) bzero(r+size,siz-size);

#ifdef DEBUG
  memop++;
  for (siz=0;siz<mtop;siz++) if (!mchunks[siz]) { mchunks[siz]=r; break; }
  if (siz==mtop) mchunks[mtop++]=r;
  if (mtop>=MAXCHUNKS) fatal("MAXCHUNKS exceeded in malloc",0);
#endif /* DEBUG */

  return r;
}


void* my_realloc(void* oldptr,const int size) {
  void* r;
  int oldsiz;

#ifdef DEBUG
  int i;
  memop++;
  if (!oldptr) return my_malloc(size);
  for (i=0;i<mtop;i++) if (mchunks[i]==oldptr) break;
  if (i==mtop) fatal("realloc on non-allocated memory",0);
#endif /* DEBUG */

  if (size<=0) fatal("my_realloc size is <=0",0);
  oldsiz=malloc_usable_size(oldptr);
  r=realloc(oldptr,size);
  if (!r) fatal("realloc failed",0);
  if (size>oldsiz) bzero(r+oldsiz,size-oldsiz);

#ifdef DEBUG
  mchunks[i]=r;
#endif /* DEBUG */

  return r;
}


void my_free(const void* r) {

#ifdef DEBUG
  int i;
  memop++;
  for (i=0;i<mtop;i++) if (mchunks[i]==r) break;
  if (i==mtop) fatal("free for non-allocated chunk",0);
  mchunks[i]=0;
#endif /* DEBUG */

  free((void*)r);

}


